@@ -1,32 +1,40 @@
+from __future__ import annotations
+
 import os
 import tempfile
+import urllib.parse
 
+from pathlib import Path
 from subprocess import CalledProcessError
-
-from clikit.api.io import IO
+from typing import TYPE_CHECKING
+from typing import Any
 
 from poetry.core.pyproject.toml import PyProjectTOML
-from poetry.repositories.pool import Pool
+from poetry.core.semver.version import Version
+
+from poetry.installation.base_installer import BaseInstaller
+from poetry.repositories.http import HTTPRepository
 from poetry.utils._compat import encode
-from poetry.utils.env import Env
-from poetry.utils.helpers import safe_rmtree
-
-from .base_installer import BaseInstaller
+from poetry.utils.helpers import remove_directory
+from poetry.utils.pip import pip_install
 
 
-try:
-    import urllib.parse as urlparse
-except ImportError:
-    import urlparse
+if TYPE_CHECKING:
+    from cleo.io.io import IO
+    from poetry.core.masonry.builders.builder import Builder
+    from poetry.core.packages.package import Package
+
+    from poetry.repositories.pool import Pool
+    from poetry.utils.env import Env
 
 
 class PipInstaller(BaseInstaller):
-    def __init__(self, env, io, pool):  # type: (Env, IO, Pool) -> None
+    def __init__(self, env: Env, io: IO, pool: Pool) -> None:
         self._env = env
         self._io = io
         self._pool = pool
 
-    def install(self, package, update=False):
+    def install(self, package: Package, update: bool = False) -> None:
         if package.source_type == "directory":
             self.install_directory(package)
 
@@ -43,35 +51,50 @@ class PipInstaller(BaseInstaller):
             package.source_type not in {"git", "directory", "file", "url"}
             and package.source_url
         ):
+            assert package.source_reference is not None
             repository = self._pool.repository(package.source_reference)
-            parsed = urlparse.urlparse(package.source_url)
+            parsed = urllib.parse.urlparse(package.source_url)
             if parsed.scheme == "http":
-                self._io.error(
-                    "    <warning>Installing from unsecure host: {}</warning>".format(
-                        parsed.hostname
-                    )
+                assert parsed.hostname is not None
+                self._io.write_error(
+                    "    <warning>Installing from unsecure host:"
+                    f" {parsed.hostname}</warning>"
                 )
                 args += ["--trusted-host", parsed.hostname]
 
-            if repository.cert:
-                args += ["--cert", str(repository.cert)]
+            if isinstance(repository, HTTPRepository):
+                certificates = repository.certificates
 
-            if repository.client_cert:
-                args += ["--client-cert", str(repository.client_cert)]
+                if certificates.cert:
+                    args += ["--cert", str(certificates.cert)]
 
-            index_url = repository.authenticated_url
+                if parsed.scheme == "https" and not certificates.verify:
+                    assert parsed.hostname is not None
+                    args += ["--trusted-host", parsed.hostname]
 
-            args += ["--index-url", index_url]
-            if self._pool.has_default():
-                if repository.name != self._pool.repositories[0].name:
+                if certificates.client_cert:
+                    args += ["--client-cert", str(certificates.client_cert)]
+
+                index_url = repository.authenticated_url
+
+                args += ["--index-url", index_url]
+
+            if (
+                self._pool.has_default()
+                and repository.name != self._pool.repositories[0].name
+            ):
+                first_repository = self._pool.repositories[0]
+
+                if isinstance(first_repository, HTTPRepository):
                     args += [
                         "--extra-index-url",
-                        self._pool.repositories[0].authenticated_url,
+                        first_repository.authenticated_url,
                     ]
 
         if update:
             args.append("-U")
 
+        req: str | list[str]
         if package.files and not package.source_url:
             # Format as a requirements.txt
             # We need to create a requirements.txt file
@@ -95,7 +118,7 @@ class PipInstaller(BaseInstaller):
 
             self.run(*args)
 
-    def update(self, package, target):
+    def update(self, package: Package, target: Package) -> None:
         if package.source_type != target.source_type:
             # If the source type has changed, we remove the current
             # package to avoid perpetual updates in some cases
@@ -103,7 +126,7 @@ class PipInstaller(BaseInstaller):
 
         self.install(target, update=True)
 
-    def remove(self, package):
+    def remove(self, package: Package) -> None:
         try:
             self.run("uninstall", package.name, "-y")
         except CalledProcessError as e:
@@ -113,84 +136,92 @@ class PipInstaller(BaseInstaller):
             raise
 
         # This is a workaround for https://github.com/pypa/pip/issues/4176
-        nspkg_pth_file = self._env.site_packages.path / "{}-nspkg.pth".format(
-            package.name
-        )
-        if nspkg_pth_file.exists():
+        for nspkg_pth_file in self._env.site_packages.find_distribution_nspkg_pth_files(
+            distribution_name=package.name
+        ):
             nspkg_pth_file.unlink()
 
         # If we have a VCS package, remove its source directory
         if package.source_type == "git":
             src_dir = self._env.path / "src" / package.name
             if src_dir.exists():
-                safe_rmtree(str(src_dir))
+                remove_directory(src_dir, force=True)
 
-    def run(self, *args, **kwargs):  # type: (...) -> str
+    def run(self, *args: Any, **kwargs: Any) -> int | str:
         return self._env.run_pip(*args, **kwargs)
 
-    def requirement(self, package, formatted=False):
+    def requirement(self, package: Package, formatted: bool = False) -> str | list[str]:
         if formatted and not package.source_type:
-            req = "{}=={}".format(package.name, package.version)
+            req = f"{package.name}=={package.version}"
             for f in package.files:
                 hash_type = "sha256"
                 h = f["hash"]
                 if ":" in h:
                     hash_type, h = h.split(":")
 
-                req += " --hash {}:{}".format(hash_type, h)
+                req += f" --hash {hash_type}:{h}"
 
             req += "\n"
 
             return req
 
         if package.source_type in ["file", "directory"]:
+            assert package.source_url is not None
             if package.root_dir:
                 req = (package.root_dir / package.source_url).as_posix()
             else:
                 req = os.path.realpath(package.source_url)
 
             if package.develop and package.source_type == "directory":
-                req = ["-e", req]
+                return ["-e", req]
 
             return req
 
         if package.source_type == "git":
-            req = "git+{}@{}#egg={}".format(
-                package.source_url, package.source_reference, package.name
+            req = (
+                f"git+{package.source_url}@{package.source_reference}"
+                f"#egg={package.name}"
             )
 
+            if package.source_subdirectory:
+                req += f"&subdirectory={package.source_subdirectory}"
+
             if package.develop:
-                req = ["-e", req]
+                return ["-e", req]
 
             return req
 
         if package.source_type == "url":
-            return "{}#egg={}".format(package.source_url, package.name)
+            return f"{package.source_url}#egg={package.name}"
 
-        return "{}=={}".format(package.name, package.version)
+        return f"{package.name}=={package.version}"
 
-    def create_temporary_requirement(self, package):
-        fd, name = tempfile.mkstemp(
-            "reqs.txt", "{}-{}".format(package.name, package.version)
-        )
+    def create_temporary_requirement(self, package: Package) -> str:
+        fd, name = tempfile.mkstemp("reqs.txt", f"{package.name}-{package.version}")
+        req = self.requirement(package, formatted=True)
+        if isinstance(req, list):
+            req = " ".join(req)
 
         try:
-            os.write(fd, encode(self.requirement(package, formatted=True)))
+            os.write(fd, encode(req))
         finally:
             os.close(fd)
 
         return name
 
-    def install_directory(self, package):
+    def install_directory(self, package: Package) -> str | int:
+        from cleo.io.null_io import NullIO
+
         from poetry.factory import Factory
-        from poetry.io.null_io import NullIO
 
+        assert package.source_url is not None
         if package.root_dir:
-            req = (package.root_dir / package.source_url).as_posix()
+            req = package.root_dir / package.source_url
         else:
-            req = os.path.realpath(package.source_url)
+            req = Path(package.source_url).resolve(strict=False)
 
-        args = ["install", "--no-deps", "-U"]
+        if package.source_subdirectory:
+            req /= package.source_subdirectory
 
         pyproject = PyProjectTOML(os.path.join(req, "pyproject.toml"))
 
@@ -199,9 +230,7 @@ class PipInstaller(BaseInstaller):
             # some versions of pip (< 19.0.0) don't understand it
             # so we need to check the version of pip to know
             # if we can rely on the build system
-            legacy_pip = self._env.pip_version < self._env.pip_version.__class__(
-                19, 0, 0
-            )
+            legacy_pip = self._env.pip_version < Version.from_parts(19, 0, 0)
 
             try:
                 package_poetry = Factory().create_poetry(pyproject.file.path.parent)
@@ -209,6 +238,7 @@ class PipInstaller(BaseInstaller):
                 package_poetry = None
 
             if package_poetry is not None:
+                builder: Builder
                 if package.develop and not package_poetry.package.build_script:
                     from poetry.masonry.builders.editable import EditableBuilder
 
@@ -230,42 +260,42 @@ class PipInstaller(BaseInstaller):
 
                     with builder.setup_py():
                         if package.develop:
-                            args.append("-e")
-
-                        args.append(req)
-
-                        return self.run(*args)
+                            return pip_install(
+                                path=req,
+                                environment=self._env,
+                                upgrade=True,
+                                editable=True,
+                            )
+                        return pip_install(
+                            path=req, environment=self._env, deps=False, upgrade=True
+                        )
 
         if package.develop:
-            args.append("-e")
+            return pip_install(
+                path=req, environment=self._env, upgrade=True, editable=True
+            )
+        return pip_install(path=req, environment=self._env, deps=False, upgrade=True)
 
-        args.append(req)
+    def install_git(self, package: Package) -> None:
+        from poetry.core.packages.package import Package
 
-        return self.run(*args)
+        from poetry.vcs.git import Git
 
-    def install_git(self, package):
-        from poetry.core.packages import Package
-        from poetry.core.vcs import Git
-
-        src_dir = self._env.path / "src" / package.name
-        if src_dir.exists():
-            safe_rmtree(str(src_dir))
-
-        src_dir.parent.mkdir(exist_ok=True)
-
-        git = Git()
-        git.clone(package.source_url, src_dir)
-
-        reference = package.source_resolved_reference
-        if not reference:
-            reference = package.source_reference
-
-        git.checkout(reference, src_dir)
+        assert package.source_url is not None
+        source = Git.clone(
+            url=package.source_url,
+            source_root=self._env.path / "src",
+            revision=package.source_resolved_reference or package.source_reference,
+        )
 
         # Now we just need to install from the source directory
-        pkg = Package(package.name, package.version)
-        pkg._source_type = "directory"
-        pkg._source_url = str(src_dir)
-        pkg.develop = package.develop
+        pkg = Package(
+            name=package.name,
+            version=package.version,
+            source_type="directory",
+            source_url=str(source.path),
+            source_subdirectory=package.source_subdirectory,
+            develop=package.develop,
+        )
 
         self.install_directory(pkg)
